function [T]=spf(varargin);

% The function SPF perform the symmetrical spectral factorization of the
% two sided 2-D polynomial.
% Syntax:
%   Q = SPF(P,[var],[n])
%            
% Where - Q is the strictly causal part - to get the anticausal: Q_anti=Q'
%         where Q_anti*Q=P
%       - P is the symmetrical polynomial (class TSP2)
%       - var is the causal variable 'u' or 'w' - (the direction of causality)
%         if omited 'u' is used
%       - n the number of Fourier points. Default value is 2^8 for both
%         variables. If a vector [pu,pw] is the argument then diferent
%         number of interpolations poins will be used for both variables.
%         !!! At all cost the numbers has to be EVEN at least.


% set default values
var='u';
np=2^2;
% --------------------------
% input diversity handling
n=nargin;
input_arg=cell(1,3);
input_arg(1:n)=varargin(1:n);
clear varargin;
% --------------------------
if ~isa(input_arg{1},'TSP2'),
      error('Wrong 1st argument. See HELP TSP2\SPF'); % quick exit
end;
if n>1 & n<=3,
	for k=2:n, % set parameters 
        if input_arg{k}=='u',
            var='u';
        elseif input_arg{k}=='w',
            var='w';
        elseif isa(input_arg{k},'double'),
            np=input_arg{k};
        else
            error('Wrong 2nd or 3rd argument. See HELP TSP2\SPF for possible syntax.');
        end;
	end;
elseif n>3,
    error('Too much arguments. See HELP TSP2\SPF for possible syntax.');
end;
% ------ end ..input diversity handling

% how many interpolation points in each variables and if the number is satisfying the demands
sp=size(np);
if ~sp(1)==1,
    error('Wrong dimension of the vector of interpolation-point numbers. Just 1x1 or 1x2 allowed.');
end;
if sp(2)==1,
    nu=np;
    nw=np;
elseif sp(2)==2,
    nu=np(1);
    nw=np(2);
else
    error('Wrong dimension of the vector of interpolation-point numbers. Just 1x1 or 1x2 allowed.');
end

Xd=input_arg{1}.d;%+[1,1]; % get the degrees of both variables

% Does we have enough interpolation-points ?

% check for variable 'u'
if Xd(1)==0, %special case
    nu_min=1;
elseif Xd(1) > 0,
    nu_min=2^ceil(log2(Xd(1))); % minimum for Radix-2 FFT   <- recommendet
else
    error('Wrong polynom-degree for in variable u.');
end;
if nu_min > nu,
    warning('Then number of interpolation points does not suffice for the first variable - u !');
end;

% check for variable 'w'
if Xd(2)==0, %special case
    nw_min=1;
elseif Xd(2) > 0,
    nw_min=2^ceil(log2(Xd(2))); % minimum for Radix-2 FFT   <- recommendet
else
    error('Wrong polynom-degree for in variable w.');
end;
if nw_min > nw,
    warning('Then number of interpolation points does not suffice for the first variable - w !');
end;
% ----- end .. Does we have enough points ?


% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
% xxx        perform the core of the algorithm           xxx
% xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

% preparing origin
K=squeeze(input_arg{1}.p.c);  % remove singleton dimensions
if Xd(1)==0, % in this case one dim to much was squeezed
    K=K';
end;
uh=ceil(Xd(1)/2); % the half-index in variable u in K
wh=ceil(Xd(2)/2); % the half-index in variable w in K


% preparing paddet matrix
H=zeros(nu,nw);
nuh=ceil(nu/2); % the half-index in variable 'u' in the paddet matrix H
nwh=ceil(nw/2); % the half-index in variable 'w' in the paddet matrix H

% re-arange half-parts of the succession to keep the symmetry
% in both variables after padding with zeros
% K=[A B]
%   [C D]

% better use CIRCFHIFT

H(1:uh+1,1:wh+1)            =K(uh+1:end,wh+1:end); % D
H(end-uh+1:end,end-wh+1:end)=K(1:uh,1:wh);         % A
H(end-uh+1:end,1:wh+1)      =K(1:uh,wh+1:end);     % B
H(1:uh+1,end-wh+1:end)      =K(uh+1:end,1:wh);     % C
% finally
% H = [D 0 C]
%     [0 0 0]
%     [B 0 A]
% ----- end of re-arange ....

% perform the logarithm in spectra
A=fft2(H);
% clear H  % to save memory
M=log(A);     % sometimes zeros appears - why?
% clear A  % to save memory
m=ifft2(M);
% clear M  % to save memory
% ----- end of perform the loga ......

% perform the docomposition
% causal in variable u

n=zeros(nu,nw);
if var=='u', % void the lower half of the variable u (the w-direction is taken thoroughly)
	if nuh==1,    % the number of all points in direction of 'u' is 1 or 2
        n=m*0.5;
    elseif nuh > 1,% the number of all points in direction of 'u' is 3 or more
        n(1:nuh+1,:)=m(1:nuh+1,:);
        %n(1:uh+1,1:wh)=m(1:uh+1,wh+1:end); % swap C <-> D  halfwise the 1st half
		%n(1:uh+1,wh+1:end)=m(1:uh+1,1:wh); % swap C <-> D  halfwise the 2nd half
		
		n(1,:)=n(1,:)*0.5;                 % 1st row
		n(nuh+1,:)=n(nuh+1,:)*0.5;           % ph+1 th row
	end;
elseif var=='w',  %void the lower half of the variable w
 	if nwh==1,    % the number of all points in direction of 'w' is 1 or 2
        n=m*0.5;
    elseif nwh>1,% the number of all points in direction of 'w' is 3 or more
        n(:,1:nwh+1)=m(:,1:nwh+1);
        
        %n(1:uh+1,1:wh)=m(1:uh+1,wh+1:end); % swap C <-> D  halfwise the 1st half
		%n(1:uh+1,wh+1:end)=m(1:uh+1,1:wh); % swap C <-> D  halfwise the 2nd half
		
		n(:,1)=n(:,1)*0.5;                 % 1st column
		n(:,nwh+1)=n(:,nwh+1)*0.5;           % (wh+1)-th column
	end;
else
    error ('Unexpected error. Causality chosen in a unpossible direction.');
end
%n=circshift(n,[0,nwh]);
% perform the exp in spectra
N=fft2(n); % clear (n);
FI=exp(N); % clear (N);
fi=ifft2(FI); % clear (FI);

% TSP2 class generator ------------------------------
% create 1-by-1-by-nu-by-nw
fi_dim4=zeros(1,1,nu,nw);
fi_dim4(1,1,:,:)=fi(:,:);
T.p = pol2(fi_dim4);
T.s = [1,1];
T.d = T.p.d;

% if var=='u',
%     T.o = [0,-nuh];
% elseif var=='w',
%     T.o = [-nwh,0];
% else
%     error('Wrong variable appeared.')
% end;

T.o = [0,0];
T.v = 'uw';
T.u = [];

%superiorto('double','pol2');
T = class(T,'tsp2');
T = tclear(T);
%T = pzer(T);
%end .. @tsp2/spf
    

